//<%STEMCELL:PerDirectoryNoticeText%>
//ALL RIGHTS RESERVED:
//====================
//The contents of this file, and associated files in this directory, are 
//Copyright (C) ThinkGeo,LLC , all rights reserved, 2003-2007.
//
//All software Source Code, Images, Database-Design and code, Graphics Design 
//and source files, and related content (collectively referred to as SOURCE) 
//are Copyright (c) ThinkGeo,LLC, 2003-2007, All Rights Reserved.
//ThinkGeo,LLC is a USA corporation at 1617 St. Andrews Dr.Suite 201,Lawrence, 
//KS 66047.
//http://ThinkGeo.com
//
//MapSuite is a Registered Trademark of ThinkGeo,LLC.
//</%STEMCELL:PerDirectoryNoticeText%>
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.Reflection;
using System.Reflection.Emit;
using System.Xml;
using System.Collections.ObjectModel;
using System.Collections.Generic;

namespace GeoCodeDesktopCSSample
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private const int INPUT_ID = 0;
        private const int INPUT_ADDRESS1 = 1;
        private const int INPUT_CITY = 2;
        private const int INPUT_STATE = 3;
        private const int INPUT_ZIPCODE = 4;


        private UsaGeocoder usaGeocoder = new UsaGeocoder();

        // hold the geocode result
        string message = "";

        #region Event handle

        private void btnSource_Click(object sender, EventArgs e)
        {
            if (this.OpenFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.CmbSource.Text = this.OpenFileDialog1.FileName;
            } 
        }

        private void btnTarget_Click(object sender, EventArgs e)
        {
            if (this.OpenFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.txtTarget.Text = this.OpenFileDialog1.FileName;
            }
        }

        private void btnErr_Click(object sender, EventArgs e)
        {
            if (this.OpenFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.txtError.Text = this.OpenFileDialog1.FileName;
            }
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            string CurrentDirectory = Application.StartupPath;
            this.CmbSource.Items.Add(@"..\BatchData\Demo1.csv");
            this.CmbSource.Items.Add(@"..\BatchData\Demo2.csv");
            this.CmbSource.Items.Add(@"..\BatchData\DemoKS.csv");
            this.CmbSource.Text = @"..\BatchData\Demo1.csv";
            this.txtTarget.Text = @"..\BatchData\output.csv";
            this.txtError.Text = @"..\BatchData\error.csv";
            this.cboState.Text = "KS";
        }

        private void mu_about_Click(object sender, EventArgs e)
        {
            AboutBox maboutbox = new AboutBox();
            maboutbox.ShowDialog();
        }

        private void btnAdvanceGeoCode_Click(object sender, EventArgs e)
        {
            usaGeocoder = new UsaGeocoder(txtDataDirectory.Text);
            usaGeocoder.Open();

            string streetAddressWithZip = string.Format("{0} {1}", txtAddr.Text, txtZip.Text);
            Collection<GeocoderMatch> geoCodeResult = usaGeocoder.Match(streetAddressWithZip, txtCities.Text, cboState.Text);
            DisplayResults(geoCodeResult);
            usaGeocoder.Close();
        }

        private void btnReverse_Click(object sender, EventArgs e)
        {
            usaGeocoder = new UsaGeocoder(this.txtDataDirectory.Text);
            usaGeocoder.Open();

            string searchText = string.Format("{0} {1}", txtLongitudes.Text, txtLatitudes.Text);
            Collection<GeocoderMatch> ReversegeoCodeResult = usaGeocoder.Match(searchText);
            DisplayResults(ReversegeoCodeResult);
            usaGeocoder.Close();
        }

        private void btnBatch_Click(object sender, EventArgs e)
        {
            FileInfo sourceFile = new FileInfo(this.CmbSource.Text);
            System.IO.StreamReader textReader = sourceFile.OpenText();
            FileInfo targetFile = new FileInfo(this.txtTarget.Text);
            StreamWriter targetStreamWriter = new StreamWriter(targetFile.Create());
            FileInfo errorFile = new FileInfo(this.txtError.Text);
            StreamWriter errorStreamWriter = new StreamWriter(errorFile.Create());
            try
            {
                UsaGeocoder geocodeEngine = new UsaGeocoder(this.txtDataDirectory.Text);
                geocodeEngine.Open();
                int totalCount = 0;
                int failureCount = 0;
                int successcount = 0;
                char delimiter = ',';
                string delimiterstr = ",";
                txtBatch.Text = "";
                while (textReader.Peek() != -1)
                {
                    string addressLine = textReader.ReadLine();
                    if ((addressLine != null && addressLine != ""))
                    {
                        totalCount += 1;
                        string[] addressInfo = addressLine.Split(delimiter);
                        Collection<GeocoderMatch> geoCodeResult = new Collection<GeocoderMatch>();
                        string addressInfoTrim = addressInfo[INPUT_ADDRESS1];
                        string address = addressInfoTrim.Trim();
                        try
                        {
                            string addressWithZip = string.Format("{0} {1}", address, addressInfo[INPUT_ZIPCODE]);
                            geoCodeResult = geocodeEngine.Match(addressWithZip, addressInfo[INPUT_CITY], addressInfo[INPUT_STATE]);
                            if (geoCodeResult.Count > 0)
                            {
                                WirteOutSucessfullGeoCode(targetStreamWriter, addressInfo[INPUT_ID], geoCodeResult, delimiterstr);
                                this.txtBatch.Text += addressInfo[INPUT_ID] + ", " + address + ", " + addressInfo[INPUT_CITY] + ", " + addressInfo[INPUT_STATE] + ", " + addressInfo[INPUT_ZIPCODE] + " " + " Was Geocoded Sucessfully" + "\r\n";
                                successcount += 1;
                            }
                            else
                            {
                                string errorMessage = addressInfo[INPUT_ID] + ", " + address + ", " + addressInfo[INPUT_CITY] + ", " + addressInfo[INPUT_STATE] + ", " + addressInfo[INPUT_ZIPCODE] + " " + "Was Geocoded Failed" + "\r\n";
                                errorStreamWriter.WriteLine(errorMessage);
                                this.txtBatch.Text += errorMessage;
                                failureCount += 1;
                            }
                        }
                        catch (Exception ex)
                        {
                            string errorMessage = addressInfo[INPUT_ID] + " ," + address + ", " + addressInfo[INPUT_CITY] + ", " + addressInfo[INPUT_STATE] + ", " + addressInfo[INPUT_ZIPCODE] + " " + ex.Message + "\r\n";
                            errorStreamWriter.WriteLine(errorMessage);
                            this.txtBatch.Text += errorMessage;
                            failureCount += 1;
                        }
                        this.Refresh();
                    }
                }

                geocodeEngine.Close();
                this.txtBatch.Text += "Total Count: " + totalCount.ToString() + "\r\n";
                this.txtBatch.Text += "Failure Count: " + failureCount.ToString() + "\r\n";
                this.txtBatch.Text += "Success Count: " + successcount.ToString() + "\r\n";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            finally
            {
                textReader.Close();
                targetStreamWriter.Close();
                errorStreamWriter.Close();
            }
        }
    
        private void btnBrowse_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog BrowseFolder = new FolderBrowserDialog();
            if (BrowseFolder.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                this.txtDataDirectory.Text = BrowseFolder.SelectedPath.ToString();
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if ((MessageBox.Show("Are you sure to Exit?", "Query", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != System.Windows.Forms.DialogResult.Yes))
            {
                e.Cancel = true;
            } 
        }

        private void btn_ViewSrcFile_Click(object sender, EventArgs e)
        {
             if(File.Exists(this.CmbSource.Text.ToString()) )
             {
                 System.Diagnostics.Process.Start(this.CmbSource.Text.ToString());
             }
             else
             {
                 MessageBox.Show("Please input soruce file!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
             }
        }

        private void btn_ViewTargetFile_Click(object sender, EventArgs e)
        {
            if (File.Exists(this.txtTarget.Text.ToString()))
            {
                System.Diagnostics.Process.Start(this.txtTarget.Text.ToString());
            }
            else
            {
                MessageBox.Show("Please input target file!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void btn_ViewErrorFile_Click(object sender, EventArgs e)
        {
            if (File.Exists(this.txtError.Text.ToString()))
            {
                System.Diagnostics.Process.Start(this.txtError.Text.ToString());
            }
            else
            {
                MessageBox.Show("Please input error file!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        #endregion

        /// <summary>
        /// Show the result
        /// </summary>
        /// <param name="GeoCodeResult">GeoCodeResult</param>
        private void DisplayResults(Collection<GeocoderMatch> geocoderResult)
        {
            // Clear the results
            lstResult.Items.Clear();
            txtResult.Text = string.Empty;

            // Load the matching items into the grid
            foreach (GeocoderMatch matchItem in geocoderResult)
            {
                if (matchItem.NameValuePairs.ContainsKey("Street"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "Street", "Zip", "State" }));
                }
            }

            // If we find addresses then select the first one to zoom in, if not then say we did not find any
            if (lstResult.Items.Count > 0)
            {
                lstResult.SelectedIndex = 0;
            }
            else
            {
                MessageBox.Show("Address can not be found!", "Geocoder Sample", MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
            }
        }

        /// <summary>
        /// Write result to targetFile
        /// </summary>
        /// <param name="TargetFile">targetFile</param>
        /// <param name="ID">ID</param>
        /// <param name="GeoCodeResult">GeoCodeResult</param>
        /// <param name="WriteHeaderRow">WriteHeaderRow</param>
        /// <param name="Delimeter">Delimeter</param>
        private void WirteOutSucessfullGeoCode(StreamWriter TargetFile, string ID, Collection<GeocoderMatch> GeoCodeResult, string Delimeter)
        {
            //PropertyInfo[] myPropertyInfo = GeoCodeResult.GetType().GetProperties();
            //string headerLine;
            string dataRow;
            //if (WriteHeaderRow)
            //{
            //    headerLine = "ID" + Delimeter;
            //    foreach (PropertyInfo myProperty1 in myPropertyInfo)
            //    {
            //        headerLine = headerLine + myProperty1.Name + Delimeter;
            //    }
            //    TargetFile.WriteLine(headerLine);
            //}
            dataRow = ID + Delimeter;
            bool isContainedStreet = false;

            foreach(GeocoderMatch matchItem in GeoCodeResult)
            {
                if(matchItem.NameValuePairs.ContainsKey("Street"))
                {
                    dataRow += "Address = " + matchItem.NameValuePairs["Street"] + "\r\n";
                    dataRow += "PrefixDirectional = " + matchItem.NameValuePairs["PrefixDirectional"] + "\r\n";
                    dataRow += "SuffixDirectional = " + matchItem.NameValuePairs["SuffixDirectional"] + "\r\n";
                    dataRow += "StreetType = " + matchItem.NameValuePairs["StreetType"] + "\r\n";
                    dataRow += "TigerLineID = " + matchItem.NameValuePairs["TigerLineID"] + "\r\n";
                    dataRow += "FromAddressLeft = " + matchItem.NameValuePairs["FromAddressLeft"] + "\r\n";
                    dataRow += "ToAddressLeft = " + matchItem.NameValuePairs["ToAddressLeft"] + "\r\n";
                    dataRow += "FromAddressRight = " + matchItem.NameValuePairs["FromAddressRight"] + "\r\n";
                    dataRow += "ToAddressRight = " + matchItem.NameValuePairs["ToAddressRight"] + "\r\n";
                    dataRow += "Zip = " + matchItem.NameValuePairs["Zip"] + "\r\n";
                    dataRow += "FromLongitude = " + matchItem.NameValuePairs["FromLongitude"] + "\r\n";
                    dataRow += "FromLatitude = " + matchItem.NameValuePairs["FromLongitude"] + "\r\n";
                    dataRow += "ToLongitude = " + matchItem.NameValuePairs["ToLongitude"] + "\r\n";
                    dataRow += "ToLatitude = " + matchItem.NameValuePairs["ToLatitude"] + "\r\n";
                    dataRow += "State = " + matchItem.NameValuePairs["State"] + "\r\n";
                    dataRow += "County = " + matchItem.NameValuePairs["County"] + "\r\n";
                    dataRow += "City = " + matchItem.NameValuePairs["City"] + "\r\n";
                    dataRow += "IsHighway = " + matchItem.NameValuePairs["IsHighway"] + "\r\n";
                    dataRow += "IsPOBox = " + matchItem.NameValuePairs["IsPOBox"] + "\r\n";
                    dataRow += "DaylightSavingTime = " + matchItem.NameValuePairs["DaylightSavingTime"] + "\r\n";
                    dataRow += "Elevation = " + matchItem.NameValuePairs["Elevation"] + "\r\n";
                    dataRow += "CityAliasAbbreviation = " + matchItem.NameValuePairs["CityAliasAbbreviation"] + "\r\n";
                    dataRow += "CityAlias = " + matchItem.NameValuePairs["CityAlias"] + "\r\n";
                    dataRow += "CountyFIPS = " + matchItem.NameValuePairs["CountyFIPS"] + "\r\n";
                    dataRow += "TimeZone = " + matchItem.NameValuePairs["TimeZone"] + "\r\n";
                    dataRow += "StateFIPS = " + matchItem.NameValuePairs["StateFIPS"] + "\r\n";
                    dataRow += "HouseNumber = " + matchItem.NameValuePairs["HouseNumber"] + "\r\n";
                    dataRow += "SideOfStreet = " + matchItem.NameValuePairs["SideOfStreet"] + "\r\n";
                    dataRow += "BoundingBox = " + matchItem.NameValuePairs["BoundingBox"] + "\r\n";
                    dataRow += "CentroidPoint = " + matchItem.NameValuePairs["CentroidPoint"] + "\r\n";

                    isContainedStreet = true;

                    TargetFile.WriteLine(dataRow);
                }
            }
            
            if(!isContainedStreet)
            {
                dataRow += "Failure: " + "Cannot find any street addresses" + "\r\n";
            }
        }

        private void lstResult_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Populate the address grid based on the selected address
            GeocoderMatch matchItem = ((ListItem)lstResult.SelectedItem).MatchItem;
            txtResult.Text = string.Empty;

            foreach (KeyValuePair<string, string> item in matchItem.NameValuePairs)
            {
                txtResult.Text += string.Format("{0}: {1}\r\n", item.Key, item.Value);
            }
        }
    }
}